library(plyr) #OJO! la llibreria plyr ha de ser cridada abans que dplyr 
library(dplyr) #per treballar amb cbinds i esas cosas
library(caret)
library(zoo)
library(scatterplot3d)
library(plotly)

#borrar
#library(RNetCDF)
library(sp)
library(sf)
library(mapview)
#library(ncdf4)
#library(rgeos) 
library(ggplot2)
#library(broom)
#library(magrittr)
#library(ggpubr)
library(raster)
#library(rgdal)
#library(tmap)
library(gstat)
library(tidyr)
#library(abind)
#library(rbind)
library(openair)
library(purrr)
library(webshot2)
library(leaflet)
library(viridisLite)


#install.packages("webshot2")
#webshot::install_phantomjs()


#sense_gbm <- list.files(path="/home/abarrant/esarchive/scratch/abarrant/results_regional_2019/results_gene_pm2p5/output/",
#pattern = ".rds$", full.names = TRUE) %>%
#map_dfr(readRDS) 

# MAP AVAILABILITY -----------------------------------------------------------------------

#/home/abarrant
results_path <- "/esarchive/scratch/abarrant/results_regional_2019/results_gene_pm2p5/output/"
save_path <- "/esarchive/scratch/abarrant/results_regional_2019/results_gene_pm2p5/stats_results/"

stations_avail <- readRDS(paste0(results_path, "df_annual_AQMS_availability_gene_pm2p5.rds"))

sample_avail <- st_as_sf(stations_avail, coords = c("longitude", "latitude"), crs = 4326)
sample_avail <- sample_avail[-31,]


map_avail <- mapview(
  sample_avail,
  zcol = "type",     
  cex = "availability",
  legend = TRUE
)
map_avail

#webshot::install_phantomjs()
mapview::mapshot(map_avail, file = paste0(save_path, "map_avail.html"))


mapview::mapshot(
  x=map_avail,
  url = NULL,
  quietly = TRUE,
  file = paste0(save_path, "map_avail.pdf"),
  remove_controls = c("zoomControl", "layersControl", "homeButton", "scaleBar",
                      "drawToolbar", "easyButton")
)

#random_day <- readRDS("/home/abarrant/esarchive/scratch/abarrant/results_regional_2019/results_gene_pm2p5/output/UK_gene_pm2p5_gbm_20190617.rds")


# GENERACIO MAPES ESTADISTICS --------------------------------------------------------------------

#COMENTARI: cal haver generat el sample_avail per tal dagafar les coordenades de cada estacio i fer els sf

df_obs <- readRDS(paste0(results_path, "df_obs_gene_pm2p5.rds"))
model_raw <- readRDS(paste0(results_path, "model_raw_at_AQMS_gene_pm2p5_df.rds"))
loocv_uk <- readRDS(paste0(results_path, "LOOCV_gene_pm2p5_df.rds"))
loocv_uk_gbm <- readRDS(paste0(results_path, "LOOCV_gene_pm2p5_gbm_df.rds"))

#seleccionem les estacions q estan presents tant a les obs com al model
common_stations <- intersect(names(df_obs), names(model_raw))
common_stations <- common_stations[common_stations!="ES2088A"] #traiem l'estacio daiguafreda, mantenim la columna de dates


#generacio dels df amb nomes les )estacions que hi ha obs
df_obs          <- df_obs[, common_stations]
df_model_raw    <- model_raw[, common_stations]
df_loocv_uk     <- loocv_uk[, common_stations]
df_loocv_uk_gbm <- loocv_uk_gbm[, common_stations]


## TABLE CONJUNT ESTACIONS I DIES -----------------------------------------------------------

df_obs_long <- gather(df_obs, key = "station", value = "conc", -date)
df_model_long <- gather(df_model_raw, key = "station", value = "conc", -date)
df_uk_long <- gather(df_loocv_uk, key = "station", value = "conc", -date)
df_uk_gbm_long <- gather(df_loocv_uk_gbm, key = "station", value = "conc", -date)

df_all_long <- data.frame(date = df_obs_long$date, station = df_obs_long$station,
                          obs = df_obs_long$conc, 
                          raw = df_model_long$conc, 
                          uk = df_uk_long$conc, 
                          uk_gbm = df_uk_gbm_long$conc)

raw_obs    = openair::modStats(df_all_long, obs = "obs", mod = "raw")
uk_obs     = openair::modStats(df_all_long, obs = "obs", mod = "uk")
uk_gbm_obs = openair::modStats(df_all_long, obs = "obs", mod = "uk_gbm")
#uk_gbm_uk  = openair::modStats(df_all_long, obs = "uk", mod = "uk_gbm")

stats_all_table <- rbind(raw_obs, uk_obs, uk_gbm_obs)  #uk_gbm_uk
row.names(stats_all_table) <- c("raw", "uk", "uk_gbm") #uk_gbm_uk

#saveRDS(object=stats_all_table, file=paste0(save_path,"stats_all_table.rds"))


### temporal variation ------------------------------------------------------------

df_eixample <- df_all_long %>%
  filter(station == "ES1438A")


pollutant <- c("obs", "raw", "uk", "uk_gbm")
time_var_eix <- openair::timeVariation(df_eixample,
                                       pollutant = pollutant,
                                       ylab = paste0(pollutant, " ", "(ug/m3)"),
                                       type = "year",
                                       key = TRUE,
                                       title = "ES1438A",
                                       #auto.text = FALSE, 
                                       ci = FALSE,
                                       #conf.int = c(0.75, 0.99),
                                       #ylim = c(0, 150),
                                       cols = c("black", "blue", "green", "red"),
                                       lty = c("dotted", "solid", "solid", "solid"),
                                       lwd = c(3, 2, 2, 2),
                                       name.pol = pollutant)


#png(file = paste0(save_path,"temporal_mean_time_VAR.png"), width = 800, height = 600)








## Estadistica per cada estacio (parametres annuals) ------------------------------------------

#common_stations[31] == "date"
for (station in common_stations[common_stations!="date"]){
  #station <- common_stations[1]
  print(station)
  
  #generem df entre les dades obs i el model (raw i corregits) per cada estacio i tots els dies de lany
  model_obs <- data.frame(date = df_obs$date, 
                          obs = df_obs[, station], 
                          model = df_model_raw[, station])
  uk_obs <- data.frame(date = df_obs$date, 
                       obs = df_obs[, station], 
                       model = df_loocv_uk[, station])
  uk_gbm_obs <- data.frame(date = df_obs$date, 
                           obs = df_obs[, station], 
                           model = df_loocv_uk_gbm[, station])
  
  #calcul parametres estadistics de cada estacio (anuals)
  stats_raw <- openair::modStats(model_obs, obs = "obs", mod = "model")
  stats_raw$stations <- station
  stats_raw$geometry <- sample_avail$geometry[which(sample_avail$ref_obs == station)]
  
  stats_uk <- openair::modStats(uk_obs, obs = "obs", mod = "model")
  stats_uk$stations <- station
  stats_uk$geometry <- sample_avail$geometry[which(sample_avail$ref_obs == station)]
  
  stats_uk_gbm <- openair::modStats(uk_gbm_obs, obs = "obs", mod = "model")
  stats_uk_gbm$stations <- station
  stats_uk_gbm$geometry <- sample_avail$geometry[which(sample_avail$ref_obs == station)]
  
  #juntem amb els estadistics de totes les estacions
  if (exists("total_annual_stats_raw")) {
    total_annual_stats_raw <- dplyr::full_join(total_annual_stats_raw, stats_raw)
  } else{
    total_annual_stats_raw <- stats_raw
  }
  
  if (exists("total_annual_stats_uk")) {
    total_annual_stats_uk <- dplyr::full_join(total_annual_stats_uk, stats_uk)
  } else{
    total_annual_stats_uk <- stats_uk
  }
  
  if (exists("total_annual_stats_uk_gbm")) {
    total_annual_stats_uk_gbm <- dplyr::full_join(total_annual_stats_uk_gbm, stats_uk_gbm)
  } else{
    total_annual_stats_uk_gbm <- stats_uk_gbm
  }
}

#convertim els df en spatial objects
total_annual_stats_raw <- st_as_sf(total_annual_stats_raw)
st_crs(total_annual_stats_raw) <- 4326

total_annual_stats_uk <- st_as_sf(total_annual_stats_uk)
st_crs(total_annual_stats_uk) <- 4326

total_annual_stats_uk_gbm <- st_as_sf(total_annual_stats_uk_gbm)
st_crs(total_annual_stats_uk_gbm) <- 4326

#guardem sf
#save_path <- "/home/abarrant/esarchive/scratch/abarrant/results_regional_2019/results_gene_pm2p5/stats_results/"
#saveRDS(object=total_annual_stats_raw, file=paste0(save_path,"total_annual_stats_raw.rds"))
#saveRDS(object=total_annual_stats_uk, file=paste0(save_path,"total_annual_stats_uk.rds"))
#saveRDS(object=total_annual_stats_uk_gbm, file=paste0(save_path,"total_annual_stats_uk_gbm.rds"))

#podem obrirlos directament si ja estan executats i guardats
total_annual_stats_raw    <- readRDS(file=paste0(save_path,"total_annual_stats_raw.rds"))
total_annual_stats_uk     <- readRDS( file=paste0(save_path,"total_annual_stats_uk.rds"))
total_annual_stats_uk_gbm <- readRDS(file=paste0(save_path,"total_annual_stats_uk_gbm.rds"))

color_palette <- colorNumeric(
  palette = "viridis", 
  domain = c(0, 1)
)


map_r_raw <- mapview(
  total_annual_stats_raw,
  zcol = "r",     
  cex = 5,
  #col.regions = color_palette,
  legend = TRUE,
  at = seq(0, 1, length.out = 6) 
)

map_r_uk <- mapview(
  total_annual_stats_uk,
  zcol = "r",     
  cex = 5,
  #col.regions = color_palette,
  legend = TRUE,
  at = seq(0, 1, length.out = 6) 
)

map_r_uk_gbm <- mapview(
  total_annual_stats_uk_gbm,
  zcol = "r",     
  cex = 5,
  #col.regions = color_palette,
  legend = TRUE,
  at = seq(0, 1, length.out = 6) 
)

map_r_raw
map_r_uk
map_r_uk_gbm



## Anomalia de r2 ------------------------------------------------------------------

#anomalia espacial de r, df conjunt, estan totes les anomalies aqui dins
anomalia_r2 <- data.frame(stations = total_annual_stats_raw$stations, 
                         r_uk_raw = (total_annual_stats_uk$r)**2 - (total_annual_stats_raw$r)**2, 
                         r_uk_gbm_raw = (total_annual_stats_uk_gbm$r)**2 - (total_annual_stats_raw$r)**2, 
                         r_uk_gbm = (total_annual_stats_uk_gbm$r)**2 - (total_annual_stats_uk$r)**2, 
                         geometry = total_annual_stats_raw$geometry)

anomalia_r2 <- st_as_sf(anomalia_r2)
st_crs(anomalia_r2) <- 4326
#saveRDS(object=anomalia_r, file=paste0(save_path,"anomaly_r2_stations.rds"))



#open
anomalia_r2 <- readRDS(paste0(save_path,"anomaly_r2_stations.rds"))

map_anomaly_r2_uk_raw <- mapview(
  anomalia_r2,
  zcol = "r_uk_raw",     
  cex = 5,   #buscar mida que es vegi be   col.regions= pal,
  legend = TRUE, 
  at = seq(-0.25, 0.65, length.out = 6) 
)

map_anomaly_r2_uk_gbm_raw <- mapview(
  anomalia_r2,
  zcol = "r_uk_gbm_raw",     
  cex = 5,   #buscar mida que es vegi be   col.regions= pal,
  legend = TRUE, 
  at = seq(-0.25, 0.65, length.out = 6) 
)

map_anomaly_r2_uk_gbm <- mapview(
  anomalia_r2,
  zcol = "r_uk_gbm",     
  cex = 5,   #buscar mida que es vegi be   col.regions= pal,
  legend = TRUE, 
  at = seq(-0.25, 0.65, length.out = 6) 
)

map_anomaly_r2_uk_raw
map_anomaly_r2_uk_gbm_raw
map_anomaly_r2_uk_gbm






## Estadistica diaria (promig conjunt d'estacions) ------------------------------------------------------


for (i in 1:nrow(df_obs)){
  #i <- 1
  
  #generem df entre les dades obs i el model (raw i corregits) per cada estacio
  model_obs <- data.frame(obs = t(df_obs[i, -31]), 
                          model = t(df_model_raw[i, -31]))
  names(model_obs) <- c("obs", "model")
  
  uk_obs <- data.frame(obs = t(df_obs[i, -31]), 
                       model = t(df_loocv_uk[i, -31]))
  names(uk_obs) <- c("obs", "model")
  
  uk_gbm_obs <- data.frame(obs = t(df_obs[i, -31]), 
                           model = t(df_loocv_uk_gbm[i, -31]))
  names(uk_gbm_obs) <- c("obs", "model")
  
  #calcul parametres estadistics de cada estacio (anuals)
  stats_raw <- openair::modStats(model_obs, obs = "obs", mod = "model")
  stats_raw$date <- df_obs$date[i]
  
  stats_uk <- openair::modStats(uk_obs, obs = "obs", mod = "model")
  stats_uk$date <- df_obs$date[i]
  
  stats_uk_gbm <- openair::modStats(uk_gbm_obs, obs = "obs", mod = "model")
  stats_uk_gbm$date <- df_obs$date[i]
  
  #juntem amb els estadistics de totes les estacions
  if (exists("total_stations_stats_raw")) {
    total_stations_stats_raw <- dplyr::full_join(total_stations_stats_raw, stats_raw)
  } else{
    total_stations_stats_raw <- stats_raw
  }
  
  if (exists("total_stations_stats_uk")) {
    total_stations_stats_uk <- dplyr::full_join(total_stations_stats_uk, stats_uk)
  } else{
    total_stations_stats_uk <- stats_uk
  }
  
  if (exists("total_stations_stats_uk_gbm")) {
    total_stations_stats_uk_gbm <- dplyr::full_join(total_stations_stats_uk_gbm, stats_uk_gbm)
  } else{
    total_stations_stats_uk_gbm <- stats_uk_gbm
  }
}

#mitjanes estadistiques per tots els dies (removing the last col which is the date)
#mean_table <- t(data.frame(raw = colMeans(total_stations_stats_raw[, 2:(ncol(total_stations_stats_raw)-1)]), 
                           #uk = colMeans(total_stations_stats_uk[, 2:(ncol(total_stations_stats_uk)-1)]), 
                           #uk_gbm = colMeans(total_stations_stats_uk_gbm[, 2:(ncol(total_stations_stats_uk_gbm)-1)])))




#guardem df
#save_path <- "/home/abarrant/esarchive/scratch/abarrant/results_regional_2019/results_gene_pm2p5/stats_results/"
#saveRDS(object=total_stations_stats_raw, file=paste0(save_path,"total_stations_stats_raw.rds"))
#saveRDS(object=total_stations_stats_uk, file=paste0(save_path,"total_stations_stats_uk.rds"))
#saveRDS(object=total_stations_stats_uk_gbm, file=paste0(save_path,"total_stations_stats_uk_gbm.rds"))
#saveRDS(object=mean_table, file=paste0(save_path,"mean_table.rds"))

total_stations_stats_raw <- readRDS(file=paste0(save_path,"total_stations_stats_raw.rds"))
total_stations_stats_uk <- readRDS(file=paste0(save_path,"total_stations_stats_uk.rds"))
total_stations_stats_uk_gbm <- readRDS(file=paste0(save_path,"total_stations_stats_uk_gbm.rds"))


conjunt_r2 <- data_frame(date = total_stations_stats_raw$date, 
                         raw = (total_stations_stats_raw$r)**2, 
                         uk = (total_stations_stats_uk$r)**2, 
                         uk_gbm = (total_stations_stats_uk_gbm$r)**2)

conjunt_r <- data_frame(date = total_stations_stats_raw$date, 
                        raw = (total_stations_stats_raw$r), 
                        uk = (total_stations_stats_uk$r), 
                        uk_gbm = (total_stations_stats_uk_gbm$r))

conjunt_r2_long <- gather(conjunt_r2, key = "variable", value = "value", -date)
ggplot(conjunt_r2_long, aes(x = date, y = value, color = variable)) +
  geom_line() + #position="dodge"
  labs(title = "Total stations",
       x = "Date (day)",
       y = expression(r^2)) +
  theme_bw() 
  


#print(summary(conjunt_r2$raw))
#print(summary(conjunt_r2$uk))
#print(summary(conjunt_r2$uk_gbm))


# CONCENTRATION MAPS AT THE STATION -----------------------------------------------------

df_mean <- data.frame(mean_conc_obs = colMeans(df_obs[, -31], na.rm = TRUE),
                      mean_conc_raw = colMeans(df_model_raw[, -31], na.rm = TRUE), 
                      mean_conc_uk = colMeans(df_loocv_uk[, -31], na.rm = TRUE), 
                      mean_conc_uk_gbm = colMeans(df_loocv_uk_gbm[, -31], na.rm = TRUE))
df_mean$stations <- rownames(df_mean)

#posar geometry a cada estacio
match_stations <- match(df_mean$stations, sample_avail$ref_obs)
df_mean$geometry <- sample_avail$geometry[match_stations]

df_anomalies_concmean <- data.frame(raw_obs = df_mean$mean_conc_raw - df_mean$mean_conc_obs, 
                                    uk_obs = df_mean$mean_conc_uk - df_mean$mean_conc_obs,
                                    uk_gbm_obs = df_mean$mean_conc_uk_gbm - df_mean$mean_conc_obs, 
                                    stations = df_mean$stations,
                                    geometry = df_mean$geometry)
  
  
#convert to sf
df_mean <- st_as_sf(df_mean)
st_crs(df_mean) <- 4326

df_anomalies_concmean <- st_as_sf(df_anomalies_concmean)
st_crs(df_anomalies_concmean) <- 4326

#saveRDS(object=df_mean, file=paste0(save_path,"df_mean_conc_at_stations.rds"))

map_conc_raw_obs <- mapview(
  df_anomalies_concmean,
  zcol = "raw_obs",     
  cex = 5,
  legend = TRUE, 
  at = seq(-15.2, 5.8, length.out = 6) 
)

map_conc_uk_obs <- mapview(
  df_anomalies_concmean,
  zcol = "uk_obs",     
  cex = 5,
  legend = TRUE, 
  at = seq(-15.2, 5.8, length.out = 6) 
)

map_conc_uk_gbm_obs <- mapview(
  df_anomalies_concmean,
  zcol = "uk_gbm_obs",     
  cex = 5,
  legend = TRUE, 
  at = seq(-15.2, 5.8, length.out = 6) 
)
map_conc_raw_obs
map_conc_uk_obs
map_conc_uk_gbm_obs





# BOXPLOTS GBM INFLUENCES ---------------------------------------------------------

influences <- readRDS("/home/abarrant/esarchive/scratch/abarrant/results_regional_2019/results_gene_pm2p5/output/gbm_influences_pm2p5.rds")

#convertir columnes en files (flatten)
influences_long <- gather(influences[, -6], key = "variable", value = "value")

#boxplot
ggplot(influences_long, aes(x = variable, y = value, fill = variable)) +
  geom_boxplot() +
  labs(title = "GBM Influences at all stations",
       x = "Predictor",
       y = "Influence (%)") +
  theme_bw() +
  theme(legend.position = "none") +
  scale_fill_viridis_d(option = "plasma")
